
# q-expansion of newform 3240.2.q.w, downloaded from the LMFDB on 21 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3240
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[2431, [1, 0]], [1621, [1, 0]], [3161, [0, -1]], [1297, [1, 0]]]
aps_data = [[0, 0], [0, 0], [0, 1], [4, -4], [-2, 2], [0, -4], [1, 0], [-5, 0], [0, -5], [-8, 8], [0, -7], [-6, 0], [0, -6], [2, -2], [-8, 8], [9, 0], [0, -4], [-13, 13], [0, 10], [-6, 0], [-6, 0], [-9, 9], [17, -17], [-6, 0], [8, -8], [-12, 12], [0, -4], [12, 0], [17, 0], [0, -6], [-18, 0], [0, -6], [-11, 11], [0, -12], [0, 14], [16, -16], [0, -22], [-2, 0], [0, 15], [-7, 7], [16, 0], [-7, 0], [26, -26], [0, -16], [-25, 0], [-4, 0], [0, -17], [-2, 2], [9, -9], [0, -19], [-6, 0], [0, -6], [1, -1], [12, 0], [0, -3], [12, -12], [-26, 0], [15, 0], [22, -22], [16, -16], [0, -14], [0, 1], [10, 0], [0, -16], [28, -28], [-33, 33], [0, 0], [0, 20], [0, 20], [-9, 9], [-34, 34], [30, 0], [4, -4], [0, 20], [13, 0], [0, -21], [6, -6], [-32, 0], [0, -30], [0, 23], [0, 6], [-15, 15], [-20, 0], [20, 0], [5, -5], [-3, 3], [0, 0], [8, -8], [6, -6], [0, -6], [31, 0], [-26, 26], [34, 0], [0, -30], [0, 1], [-27, 0], [0, 28], [14, 0], [-8, 0], [22, 0], [10, -10], [22, 0], [0, 4], [-28, 28], [0, 5], [-10, 0], [-33, 33], [15, 0], [0, -36], [47, -47], [0, -30], [0, 0], [0, 27], [28, -28], [1, 0], [48, -48], [0, -12], [33, 0], [0, -25], [0, 0], [0, 18], [36, -36], [34, -34], [-9, 0], [-17, 17], [10, 0], [46, -46], [-34, 0], [4, -4], [0, -24], [9, 0], [0, 48], [0, 39], [38, 0], [0, 24], [0, 35], [33, 0], [0, -38], [0, 21], [-18, 0], [-28, 0], [-2, 2], [0, 50], [-3, 0], [34, 0], [-28, 28], [-32, 32], [-45, 45], [0, 31], [-39, 0], [0, 14], [-24, 0], [-4, 0], [0, -17], [12, -12], [-32, 32], [-40, 0], [-38, 38], [28, 0], [0, -38], [-11, 11], [-6, 0], [0, 42], [-26, 0], [0, 38], [27, -27], [5, 0], [-42, 42], [-25, 0], [0, 5], [44, -44], [0, -7], [0, -52], [4, -4], [0, -5], [0, 60], [-55, 55], [-32, 0], [-62, 0], [62, -62], [-2, 2], [60, -60], [0, 22], [26, 0], [0, -19], [-4, 4], [-16, 0], [28, -28], [0, 43], [-40, 0], [14, 0], [-9, 9], [53, 0], [-40, 40], [-25, 0], [0, -7], [0, -33], [-12, 12], [26, -26], [-8, 0], [0, 66], [8, -8], [0, -14], [45, -45], [0, 0], [-2, 0], [-32, 0], [0, -11], [60, -60], [0, 4], [-44, 0], [0, -70], [16, -16], [-12, 12], [0, -44], [15, -15], [0, -58], [8, -8], [-39, 0], [0, 43], [0, -26], [0, 41], [0, -32], [-18, 0], [0, 21], [5, -5], [17, -17], [10, 0], [34, -34], [-28, 28], [0, 26], [-57, 0], [0, -32], [0, 58], [24, -24], [-27, 27], [0, -21], [-9, 0], [0, 10], [30, 0], [4, -4], [-53, 0], [0, 2], [-23, 0], [0, 14], [14, -14], [52, 0], [0, -4], [0, -4], [-29, 0], [0, -30], [-34, 0], [0, 33], [-29, 29], [-61, 61], [-48, 0], [-43, 0], [-12, 12], [-27, 0], [-12, 0], [-22, 22], [-1, 1], [0, -15], [-4, 0], [0, -67], [-28, 28], [8, 0], [42, -42], [0, -42], [0, -38], [0, 37], [-2, 0], [18, -18], [0, -47], [0, -50], [44, 0], [0, 7], [31, -31], [-55, 0], [-78, 78], [0, 0], [0, -75], [11, -11], [-31, 31], [0, 68], [32, 0], [38, 0], [0, 42], [-21, 21], [-24, 0], [-56, 56], [81, 0], [0, 42], [0, 42], [38, -38], [0, 24], [25, -25], [6, -6], [-16, 0], [-24, 24], [0, -54], [21, 0], [-16, 0]]
